{% extends "!layout.html" %}

{% block extrahead %}
<style>
  body {
    font-size: 16px;
  }

  div.topic {
    padding: 0;
    background-color: #F8F4A6;
    border: 1px solid #D8D486;
  }

  div.topic p.first {
    margin: 0;
    padding: 0.1em 0 0.1em 0.5em;
    background-color: #E8E496;
    border-bottom: 1px solid #C8C476;
  }

  div.topic p {
    margin: 0.5em 1em 0.5em 1em;
  }

  p.topic-title {
    display: run-in;
    margin-right: 15px;
  }

  a.external-link {
    background: url(_static/external.png) center right no-repeat;
    padding-right: 13px;
  }

  div.related ul li a.external-link {
    padding-right: 13px;
  }

  .sidebartext {
    font-size: 75%;
    line-height: 130%;
  }

</style>

{% if include_analytics %}
<script type="text/javascript">

  var _gaq = _gaq || [];
  _gaq.push(['_setAccount', 'UA-29887834-3']);
  _gaq.push(['_setDomainName', 'pydoit.org']);
  _gaq.push(['_trackPageview']);

  (function() {
    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
  })();
</script>
{% endif %}
{% endblock %}


{% block rootrellink %}
  <li><a href="{{ pathto('index') }}">Home</a>&nbsp;|&nbsp;</li>
  <li><a href="{{ pathto('contents') }}">Documentation</a>&nbsp;|&nbsp;</li>
  <!-- TODO news -->
  <li><a class="external-link" href="https://github.com/pydoit/doit">Code</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="https://github.com/pydoit/doit/issues">Issues</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="http://pypi.python.org/pypi/doit">Download</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="http://groups.google.co.in/group/python-doit">Forum</a>&nbsp;|&nbsp;</li>
  <li><a class="external-link" href="https://twitter.com/py_doit">Twitter</a></li>
{% endblock %}


{% block header %}
<div style="background-color: white; text-align: left; padding: 10px 10px 15px 50px">
<img src="{{ pathto("_static/doit.png", 1) }}" alt="doit logo" />
</div>
{% endblock %}


{% block sidebarsearch %}
{{ super() }}

<h3>Sponsors</h3>
<em>Your logo here</em>

<h3>Sponsor/Donate</h3>

{% if include_gittip %}
<div data-gratipay-username="schettino72" data-gratipay-widget="custom">
  <div><a class="gratipay-profile-link">Gratipay</a> weekly funding goal is <span class="gratipay-goal">$?.??</span> USD.
  </div>
  <div style="position:relative;background-color:#2F2D2D;color:#DDDDDD;border-radius:5px;margin:3px 5px;padding:3px 0;">
    <div class="gratipay-goal-progress-bar" style="position:absolute;top:3px;background-color:#008000;height:24px;border-radius:2px;"></div>
    <span class="gratipay-receiving" style="position:relative;padding-left:6px;">$?.??</span>
  </div>
</div>
<script src="//grtp.co/v1.js"></script>
{% endif %}
<br/>

<div class="sidebartext">
If you use <em>doit</em> and think it is a useful project. Please consider to make a donation to support the developer/maintainer with maintanance tasks (bug fixes, merge patches, reply users) and further development.<br/>
  <script data-gratipay-username="schettino72"
        data-gratipay-widget="button"
        src="//gttp.co/v1.js"></script>
<br/>
If you use it at work, ask your company to sponsor the project. The logo of your company will be placed on this website side-bar (see above). For more information please contact <em>schettino72</em>&#64<span>gmail.com</span><br/>
</div>

<h3>Hire me</h3>
<div class="sidebartext">
Looking for a python developer with a proven record of designing and developing
robust and well tested applications?
The creator and maintainer of <em>doit</em> is available for hire.
Full-time, part-time or one-off job.
<br/>
Contact: <em>schettino72</em>&#64<span>gmail.com</span>
</div>



{# logo - python #}
<h3></h3>
<a href="http://python.org"><img src="_static/python-powered-w-100x40.png" /></a><br/><br/>
{% endblock %}

{% block sidebarrel %}
{% endblock %}

